#include "Interface.h"

#define SCL_PIN       7
#define SCL_PORT      PORTD
#define SDA_PIN       6
#define SDA_PORT      PORTD

#define ADD_IO_EXPANDER  0x41
#define ADD_MAZET        0x74

#include <SoftWire.h>
SoftWire __WIRE = SoftWire();

uint8_t _registerValue = 0, _data[2] = {0, 0};
int16_t _i2cReadTimeouts = 0;
unsigned long _frq = 0;

controlRegisterBank   currentRegister;
io_expander           currentDevice;
integrationTime       currentTime;
attenuation           currentAttenuation;
osrRegisterState      currentStateDevice;

static void processCommand(int16_t* command);
static void toSwitchRelay();
static void toSwitchAttenuation();

void initDevice(void) {

  currentDevice =       (io_expander)IO_VFC;
  currentAttenuation =  (attenuation)ATTENUATION_1;
  currentStateDevice =  (osrRegisterState)CONFIG;
  currentTime =         (integrationTime)TINT_MS_64;

  __WIRE.begin();

  __WIRE.beginTransmission(ADD_IO_EXPANDER);
  __WIRE.write(0x03);
  __WIRE.write(0x00);
  __WIRE.endTransmission(true);

  toSwitchRelay();
  toSwitchAttenuation();
}
static void toSwitchRelay() {
  uint8_t i = 0;

  switch (currentDevice)
  {
    case IO_VFC:
      digitalWrite(SW_EN, HIGH);
      __WIRE.beginTransmission(ADD_IO_EXPANDER);
      __WIRE.write(0x01);
      __WIRE.write(0x03);
      Serial.println("VFC-Switch");
      break;
    case IO_MAZET:
      digitalWrite(SW_EN, LOW);
      __WIRE.beginTransmission(ADD_IO_EXPANDER);
      __WIRE.write(0x01);
      __WIRE.write(0x0C);
      Serial.println("MAZET-Switch");
      break;
    default:
      break;
  }

  while (i < 20)
  {
    i++;
    delay(1);
  }
  __WIRE.write(0x01);
  __WIRE.write(0x00);
  __WIRE.endTransmission(true);
}
static void toSwitchAttenuation() {
  switch (currentAttenuation)
  {
    case ATTENUATION_1:                     // 100µA
      digitalWrite(SW_A0, LOW);
      digitalWrite(SW_A1, LOW);
      Serial.println("1");
      break;
    case ATTENUATION_10:                    // 1mA
      digitalWrite(SW_A0, LOW);
      digitalWrite(SW_A1, HIGH);
      Serial.println("2");
      break;
    case ATTENUATION_100:                   // 10mA
      digitalWrite(SW_A0, HIGH);
      digitalWrite(SW_A1, LOW);
      Serial.println("3");
      break;
    case ATTENUATION_1000:                  // 100mA
      digitalWrite(SW_A0, HIGH);
      digitalWrite(SW_A1, HIGH);
      Serial.println("4");
      break;
    default:
      digitalWrite(SW_A0, LOW);             // 100µA
      digitalWrite(SW_A1, LOW);
      break;
  }
  _timetowait = 100;
}

// Event Funktionen I2C
void getDatafromDevice(void) {
  uint8_t  i = 0;

  if (currentStateDevice == CONFIG)
  {
    if (currentDevice == IO_MAZET)
    {
      _i2cReadTimeouts = 0;

      __WIRE.requestFrom(ADD_MAZET, 1);

      while (__WIRE.available() < 1)
      {
        _i2cReadTimeouts++;
        if (_i2cReadTimeouts > 100)
          return;
        else
          delay(1);
      }

      Wire.write(__WIRE.read());
    }

    if (currentRegister == AGEN)
      Wire.write(_registerValue);
    else {
      Wire.write(_data, 2);
    }
  }
  else if (currentStateDevice == MEASUREMENT)
  {
    Wire.write(_data, 2);
  }
}

void checkForCommand(int numBytes) {
  uint8_t i = 0;
  int16_t currentCommand[8] = { -1, -1, -1, -1, -1, -1, -1, -1};

  while (0 < Wire.available())
  {
    currentCommand[i] = Wire.read();
    i++;
  }
  processCommand(currentCommand);
}

static void processCommand(int16_t* command) {
  uint8_t i = 0, range = 0, tmp_attenuation = 0, tmp_device = 0;

  switch ((controlRegisterBank)command[(uint8_t)REG])
  {
    case OSR:
      {
        currentRegister = (controlRegisterBank)OSR;

        if (currentStateDevice == CONFIG)
        {
          if (command[(uint8_t)VALUE] == (uint8_t)MEASUREMENT)
          {
            currentStateDevice = (osrRegisterState)MEASUREMENT;
            _startMeasurement = true;
          }
        }
        else if (currentStateDevice == MEASUREMENT)
        {
          if (!_startMeasurement)
          {
            if (command[(uint8_t)VALUE] == (uint8_t)CONFIG)
            {
              currentStateDevice = (osrRegisterState)CONFIG;

              if (currentDevice == IO_MAZET)
              {
                __WIRE.beginTransmission(ADD_MAZET);
                __WIRE.write((uint8_t)OSR);
                __WIRE.write((uint8_t)CONFIG);
                __WIRE.endTransmission(true);
              }
            }
            else if (command[(uint8_t)VALUE] == (uint8_t)MEASUREMENT)
            {
              _startMeasurement = true;
            }
          }
        }
      }
      break;
    case AGEN:
      {
        currentRegister = (controlRegisterBank)AGEN;
        _registerValue = 0;
      }
      break;
    case CREGL:
      {
        uint8_t i = 0;

        currentRegister = (controlRegisterBank)CREGL;

        if (currentStateDevice = CONFIG)
        {
          if (command[(uint8_t)VALUE] != -1) // else read command just for mazet
          {
            tmp_device = (uint8_t)currentDevice;

            _numofiterations = (command[6] << 8);
            _numofiterations += command[7];

            range = (command[(uint8_t)VALUE] & 0xF0);
            range = range >> 4;

            if (range > 4)
            {
              currentDevice = (io_expander)IO_VFC;

              tmp_attenuation = (uint8_t)currentAttenuation;

              currentAttenuation = (attenuation)(range - (uint8_t)ATTENUATION_COUNT - 1);

              if ((uint8_t)currentAttenuation != tmp_attenuation)
                toSwitchAttenuation();
            }
            else
            {
              currentDevice = (io_expander)IO_MAZET;

              command[6] = -1;
              command[7] = -1;

              command[(uint8_t)VALUE] = (command[(uint8_t)VALUE] | 0x80);     // Setzt den Eingang von Mazet auf Anode

              __WIRE.beginTransmission(ADD_MAZET);

              while (true)
              {
                if (command[i] == -1)
                  break;

                __WIRE.write(command[i]);
                i++;
              }

              __WIRE.endTransmission(true);
            }

            // Integrationstime
            currentTime = (integrationTime)(command[(uint8_t)VALUE] & 0x0F);

            //
            if ((uint8_t)currentDevice != tmp_device)
              toSwitchRelay();
          }
          else
          {
            if (currentDevice == IO_MAZET)
            {
              __WIRE.beginTransmission(ADD_MAZET);
              __WIRE.write((uint8_t)CREGL);
              __WIRE.endTransmission(true);
            }
          }
        }
      }
      break;
    case CREGH:
      {
        currentRegister = (controlRegisterBank)CREGH;

        if (currentStateDevice = CONFIG)
        {
          if (currentDevice == IO_MAZET)
          {
            if (command[(uint8_t)VALUE] == -1) // else read command just for mazet
            {
              __WIRE.beginTransmission(ADD_MAZET);
              __WIRE.write((uint8_t)CREGH);
              __WIRE.endTransmission(true);
            }
            else                              // else write commnad just for mazet
            {
              __WIRE.beginTransmission(ADD_MAZET);
              __WIRE.write((uint8_t)CREGH);
              __WIRE.write((uint8_t)VALUE);
              __WIRE.endTransmission(true);
            }
          }
        }
      }
      break;
    case OPTREG:
      {
        currentRegister = (controlRegisterBank)OPTREG;

        if (currentStateDevice = CONFIG)
        {
          if (currentDevice == IO_MAZET)
          {
            if (command[(uint8_t)VALUE] == -1) // else read command just for mazet
            {
              __WIRE.beginTransmission(ADD_MAZET);
              __WIRE.write((uint8_t)OPTREG);
              __WIRE.endTransmission(true);
            }
            else                              // else write commnad just for mazet
            {
              __WIRE.beginTransmission(ADD_MAZET);
              __WIRE.write((uint8_t)OPTREG);
              __WIRE.write((uint8_t)VALUE);
              __WIRE.endTransmission(true);
            }
          }
        }
      }
      break;
    case BREAK:
      {
        currentRegister = (controlRegisterBank)BREAK;

        if (currentStateDevice = CONFIG)
        {
          if (currentDevice == IO_MAZET)
          {
            if (command[(uint8_t)VALUE] == -1) // else read command just for mazet
            {
              __WIRE.beginTransmission(ADD_MAZET);
              __WIRE.write((uint8_t)BREAK);
              __WIRE.endTransmission(true);
            }
            else                              // else write commnad just for mazet
            {
              __WIRE.beginTransmission(ADD_MAZET);
              __WIRE.write((uint8_t)BREAK);
              __WIRE.write((uint8_t)VALUE);
              __WIRE.endTransmission(true);
            }
          }
        }
      }
      break;
    case EDGES:
      {
        currentRegister = (controlRegisterBank)EDGES;

        if (currentStateDevice = CONFIG)
        {
          if (currentDevice == IO_MAZET)
          {
            if (command[(uint8_t)VALUE] == -1) // else read command just for mazet
            {
              __WIRE.beginTransmission(ADD_MAZET);
              __WIRE.write((uint8_t)EDGES);
              __WIRE.endTransmission(true);
            }
            else                              // else write commnad just for mazet
            {
              __WIRE.beginTransmission(ADD_MAZET);
              __WIRE.write((uint8_t)EDGES);
              __WIRE.write((uint8_t)VALUE);
              __WIRE.endTransmission(true);
            }
          }
        }
      }
      break;
    /***************************************************************
    ** Unknown Register
    ***************************************************************/
    default:
      {
        currentRegister = (controlRegisterBank)DEF;
      }
      break;
  }
}

void startIntegration() {
  uint8_t  f_compensation = 8;
  uint16_t int_time = 0, count = 0, raw_counts = 0;

  digitalWrite(INT_READY, LOW);

  if (currentDevice == IO_VFC)
  {
    int_time = (uint16_t)(pow(2.0, (double)currentTime) + 0.5);

    while (count < _numofiterations)
    {
      count++;

      FreqCounter::f_comp = f_compensation;
      FreqCounter::start(int_time);

      while (FreqCounter::f_ready == 0);

      if ( count == 1)
        _frq = FreqCounter::f_freq;
      else
        _frq = (_frq + FreqCounter::f_freq) / 2;
    }
  }
  else if (currentDevice == IO_MAZET)
  {
    while (count < _numofiterations)
    {
      count++;

      __WIRE.beginTransmission(ADD_MAZET);

      __WIRE.write((uint8_t)OSR);
      __WIRE.write((uint8_t)MEASUREMENT);

      __WIRE.endTransmission(true);

      while (mazetisnotready);

      __WIRE.beginTransmission(ADD_MAZET);
      __WIRE.write((uint8_t)0x03);  // read data from Channel 4

      _i2cReadTimeouts = 0;

      __WIRE.requestFrom(ADD_MAZET, 2);

      while (__WIRE.available() < 2)
      {
        _i2cReadTimeouts++;
        if (_i2cReadTimeouts > 100)
          return;
        else
          delay(1);
      }

      raw_counts =  __WIRE.read();
      raw_counts += (__WIRE.read() << 8);

      if (count == 1)
        _frq = raw_counts;
      else
      {
        _frq = (_frq + raw_counts) / 2;
        delay(1);
      }

      mazetisnotready = true;
    }
  }

  _data[0] = (_frq & 255);
  _data[1] = (_frq >> 8);

  _frq = 0;

  digitalWrite(INT_READY, HIGH);
  _startMeasurement = false;
}

void changeStateIntReadyPin(void) {
  mazetisnotready = false;
}
