
/***************************************************************
 **
 ** Includes.
 **
***************************************************************/
#include <Wire.h>

#include "FreqCounter.h"
#include "Interface.h"

#define ADDRESS_SLAVE_BASIS  0x74

/* Coding Address
   D1 D0| Y
  ------|---
   0  0 | 0
   0  1 | 1
   1  0 | 2
   1  1 | 3

  ADDRESS_SLAVE_BASIS + Y = ADDRESS_BOARD_I2C
*/

#define D0              11
#define D1              13


#define SW_EN           A2
#define SW_A0           A1
#define SW_A1           A0

#define READY           2
#define INT_READY       10

/***************************************************************
 **  setup()
 **
 **  Initialize the Arduino and setup serial communication.
 **
 **  Input:  None
 **  Output: None
***************************************************************/

volatile bool _startMeasurement = false, mazetisnotready = true;
volatile uint8_t _timetowait = 1;
volatile uint16_t _numofiterations = 1;

static uint8_t _add_offset = 0;

void setup()
{
  Serial.begin(115200);

  pinMode(D0, INPUT);
  pinMode(D1, INPUT);

  pinMode(SW_EN, OUTPUT);
  pinMode(SW_A0, OUTPUT);
  pinMode(SW_A1, OUTPUT);
  pinMode(INT_READY, OUTPUT);

  digitalWrite(INT_READY, HIGH);
  
  attachInterrupt(digitalPinToInterrupt(READY), changeStateIntReadyPin, RISING);
}

void loop() {

  delay(5000);
  
  if (digitalRead(D0) == HIGH) {
    Serial.println("DO = HIGH");
    _add_offset += 1;
  }
  else
    Serial.println("DO = LOW");

  if (digitalRead(D1) == HIGH) {
    Serial.println("D1 = HIGH");
    _add_offset += 2;
  }
  else
    Serial.println("D1 = LOW");

  Serial.println(_add_offset + ADDRESS_SLAVE_BASIS);

  initDevice();

  Wire.begin(_add_offset + ADDRESS_SLAVE_BASIS);

  Wire.onReceive(checkForCommand);
  Wire.onRequest(getDatafromDevice);


  // ENDLESS LOOP
  while (true) {
    if (_startMeasurement) {
      delay(_timetowait);
      startIntegration();
      _timetowait = 1;
    }
  }
}
